# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

"""This demo shows the latest notification from a connected Apple device on the EInk screen."""

import adafruit_ble
from adafruit_ble.advertising.standard import SolicitServicesAdvertisement
from adafruit_ble_apple_notification_center import AppleNotificationCenterService
from adafruit_display_ble_status.advertising import AdvertisingWidget
from adafruit_gizmo import eink_gizmo
from adafruit_display_notification import apple
from adafruit_display_notification import NotificationFree

# This is a whitelist of apps to show notifications from.
APPS = ["com.tinyspeck.chatlyio", "com.atebits.Tweetie2"]


def find_connection():
    for connection in radio.connections:
        if AppleNotificationCenterService not in connection:
            continue
        if not connection.paired:
            connection.pair()
        return connection, connection[AppleNotificationCenterService]
    return None, None


# Start advertising before messing with the display so that we can connect immediately.
radio = adafruit_ble.BLERadio()
a = SolicitServicesAdvertisement()
a.solicited_services.append(AppleNotificationCenterService)

display = eink_gizmo.EInk_Gizmo()

radio_widget = AdvertisingWidget(radio.name, display.width, display.height)
display.show(radio_widget)

# True when the screen reflects our current state.
screen_updated = False
latest_notification = None
active_connection, notification_service = find_connection()
while True:
    if not active_connection:
        radio.start_advertising(a)
        screen_updated = False

    while not active_connection:
        active_connection, notification_service = find_connection()

        if not screen_updated and display.time_to_refresh == 0:
            display.refresh()
            screen_updated = True

    while active_connection.connected:
        remaining_time = 600
        if not screen_updated:
            remaining_time = display.time_to_refresh
        new_notification = None
        for notification in notification_service.wait_for_new_notifications(
            remaining_time
        ):
            # Filter notifications we don't care about.
            if APPS and notification.app_id not in APPS:
                continue
            # For now, use _raw_date even though we should use a parsed version of the date.
            # pylint: disable=protected-access
            # Ignore notifications older than the currently shown one.
            if (
                latest_notification
                and notification._raw_date < latest_notification._raw_date
            ):
                continue
            new_notification = notification
            break

        if new_notification:
            print(new_notification)
            latest_notification = new_notification
            screen_updated = False
            display.show(
                apple.create_notification_widget(
                    latest_notification, display.width, display.height
                )
            )
        elif latest_notification and latest_notification.removed:
            # Stop showing the latest and show that there are no new notifications.
            latest_notification = None
            screen_updated = False
            display.show(NotificationFree(display.width, display.height))

        # Do not refresh the screen more often than every 180 seconds for eInk displays! Rapid
        # refreshes will damage the panel.
        if not screen_updated and display.time_to_refresh == 0:
            display.refresh()
            screen_updated = True

    active_connection = None
    notification_service = None



# SPDX-FileCopyrightText: 2019 Scott Shawcroft for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_display_notification.apple`
================================================================================

Maps Apple Notification Center Notification objects to the notification widgets
in this library.

"""

from . import PlainNotification

__version__ = "0.0.0-auto.0"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_Display_Notification.git"


def create_notification_widget(
    notification, max_width, max_height, *, color_count=2**16
):
    """Creates a notification widget for the given Apple notification."""
    # pylint: disable=unused-argument
    return PlainNotification(
        notification.title, notification.message, max_width, max_height
    )



# SPDX-FileCopyrightText: 2021 ladyada for Adafruit Industries
# SPDX-License-Identifier: MIT

"""This demo shows the latest notification from a connected Apple device on a TFT Gizmo screen.

The A and B buttons on the CircuitPlayground Bluefruit can be used to scroll through all active
notifications.
"""

import time
import board
import digitalio
import displayio
import adafruit_ble
from adafruit_ble.advertising.standard import SolicitServicesAdvertisement

from adafruit_ble_apple_notification_center import AppleNotificationCenterService
from adafruit_display_ble_status.advertising import AdvertisingWidget
from adafruit_gizmo import tft_gizmo
from adafruit_display_notification import apple
from adafruit_display_notification import NotificationFree

# from adafruit_circuitplayground import cp

# This is a whitelist of apps to show notifications from.
# APPS = ["com.tinyspeck.chatlyio", "com.atebits.Tweetie2"]
APPS = []

DELAY_AFTER_PRESS = 15
DEBOUNCE = 0.1

a = digitalio.DigitalInOut(board.BUTTON_A)
a.switch_to_input(pull=digitalio.Pull.DOWN)
b = digitalio.DigitalInOut(board.BUTTON_B)
b.switch_to_input(pull=digitalio.Pull.DOWN)


def find_connection():
    for connection in radio.connections:
        if AppleNotificationCenterService not in connection:
            continue
        if not connection.paired:
            connection.pair()
        return connection, connection[AppleNotificationCenterService]
    return None, None


# Start advertising before messing with the display so that we can connect immediately.
radio = adafruit_ble.BLERadio()
advertisement = SolicitServicesAdvertisement()
advertisement.solicited_services.append(AppleNotificationCenterService)

SCALE = 2

display = tft_gizmo.TFT_Gizmo()
group = displayio.Group(scale=SCALE)
display.show(group)

width = display.width // SCALE
height = display.height // SCALE

radio_widget = AdvertisingWidget("CIRCUITPY", width, height)
group.append(radio_widget)

current_notification = None
all_ids = []
last_press = time.monotonic()
active_connection, notification_service = find_connection()
while True:
    if not active_connection:
        radio.start_advertising(advertisement)

    while not active_connection:
        active_connection, notification_service = find_connection()

    while active_connection.connected:
        all_ids.clear()
        current_notifications = notification_service.active_notifications
        for notification_id in current_notifications:
            notification = current_notifications[notification_id]
            if APPS and notification.app_id not in APPS:
                continue
            all_ids.append(notification_id)

        # For now, use _raw_date even though we should use a parsed version of the date.
        # pylint: disable=protected-access
        all_ids.sort(key=lambda x: current_notifications[x]._raw_date)

        if current_notification and current_notification.removed:
            # Stop showing the latest and show that there are no new notifications.
            current_notification = None

        if not current_notification and not all_ids:
            group[0] = NotificationFree(width, height)
        elif all_ids:
            now = time.monotonic()
            if (
                current_notification
                and current_notification.id in all_ids
                and now - last_press < DELAY_AFTER_PRESS
            ):
                index = all_ids.index(current_notification.id)
            else:
                index = len(all_ids) - 1
            if now - last_press >= DEBOUNCE:
                if b.value and index > 0:
                    last_press = now
                    index += -1
                if a.value and index < len(all_ids) - 1:
                    last_press = now
                    index += 1

            notification_id = all_ids[index]
            if not current_notification or current_notification.id != notification_id:
                current_notification = current_notifications[notification_id]
                print(current_notification._raw_date, current_notification)
                group[0] = apple.create_notification_widget(
                    current_notification, width, height
                )

    active_connection = None
    notification_service = None



# SPDX-FileCopyrightText: 2019 Scott Shawcroft for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_display_notification`
================================================================================

Very basic notification widgets.

"""

import displayio

from adafruit_display_text import label

import terminalio

__version__ = "0.0.0-auto.0"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_Display_Notification.git"

TEXT_FONT = terminalio.FONT

# pylint: disable=too-few-public-methods


class NotificationFree(displayio.Group):
    """Widget to show when no notifications are active."""

    def __init__(self, width, height, *, dark_mode=True):
        # pylint: disable=unused-argument
        super().__init__()

        if dark_mode:
            text_color = 0xFFFFFF
        else:
            text_color = 0x000000

        # Create the text label
        self.title = label.Label(
            TEXT_FONT, text="None!", y=height // 2, color=text_color
        )
        self.append(self.title)


class PlainNotification(displayio.Group):
    """Plain text widget with a title and message."""

    def __init__(self, title, message, width, height, *, dark_mode=True):
        super().__init__()

        # Set text, font, and color
        if dark_mode:
            text_color = 0xFFFFFF
        else:
            text_color = 0x000000

        # Create the text label
        self.title = label.Label(TEXT_FONT, text=title, color=text_color, y=8)
        self.append(self.title)

        # TODO: Move this into Label or a TextBox.
        lines = PlainNotification._wrap_nicely(message, width // 7)
        max_lines = height // 20
        message = "\n".join(lines[:max_lines])

        self.message = label.Label(
            terminalio.FONT, text=message, color=text_color, x=2, y=height // 2 + 8
        )
        self.append(self.message)

    # cribbed from pyportal
    @staticmethod
    def _wrap_nicely(string, max_chars):
        """A helper that will return a list of lines with word-break wrapping.
        :param str string: The text to be wrapped.
        :param int max_chars: The maximum number of characters on a line before wrapping.
        """
        string = string.replace("\n", "").replace("\r", "")  # strip confusing newlines
        words = string.split(" ")
        the_lines = []
        the_line = ""
        for w in words:
            if len(the_line + " " + w) <= max_chars:
                the_line += " " + w
            else:
                the_lines.append(the_line)
                the_line = "" + w
        if the_line:  # last line remaining
            the_lines.append(the_line)
        # remove first space from first line:
        the_lines[0] = the_lines[0][1:]
        return the_lines
